@@ -5,26 +5,17 @@ from dateutil.tz import tzutc
 import mimetypes
 import logging
 import time
+from a2ml.api.utils import retry_helper
 
 
 def retry_handler(decorated):
     def wrapper(self, *args, **kwargs):
-        nTry = 0
-        while nTry < 10:
-            try:
-                return decorated(self, *args, **kwargs)
-            except Exception as exc:
-                if "InvalidAccessKeyId" in str(exc):
-                    logging.info("BotoClient: InvalidAccessKeyId error.Sleep and try again. Num try: %s"%nTry)
-                    nTry += 1
-                    time.sleep(10*nTry) 
-                else:
-                    raise                
-                
+        return retry_helper(lambda: decorated(self, *args, **kwargs), ['InvalidAccessKeyId'])
+
     return wrapper
 
 class BotoClient:
-    def __init__(self):
+    def __init__(self, region=None):
         import boto3
 
         endpoint_url = os.environ.get('S3_ENDPOINT_URL')
@@ -34,12 +25,14 @@ class BotoClient:
             self.client = boto3.client(
                 's3',
                 endpoint_url=endpoint_url,
-                config=boto3.session.Config(signature_version='s3v4')
+                config=boto3.session.Config(signature_version='s3v4'),
+                region_name=region,
             )
         else:
             self.client = boto3.client(
                 's3',
-                config=boto3.session.Config(signature_version='s3v4')
+                config=boto3.session.Config(signature_version='s3v4'),
+                region_name=region,
             )
 
     @retry_handler
@@ -62,6 +55,32 @@ class BotoClient:
     def create_bucket(self, *args, **kwargs):
         return self.client.create_bucket(*args, **kwargs)
 
+    @retry_handler
+    def delete_bucket(self, Bucket):
+        self._delete_items(Bucket, "list_objects", "Contents")
+        self._delete_items(Bucket, "list_object_versions", "Versions")
+        self._delete_items(Bucket, "list_object_versions", "DeleteMarkers")
+
+        self.client.delete_bucket(Bucket=Bucket)
+
+    def _map_version_key(self, obj):
+        res = {"Key": obj["Key"]}
+
+        if "VersionId" in obj:
+            res["VersionId"] = obj["VersionId"]
+
+        return res
+
+    def _delete_items(self, bucket, method_name, resource_name):
+        batch_delete_limit = 1000 # delete_objects can't delete more that 1000 keys at once
+        list_func = lambda: getattr(self.client, method_name)(Bucket=bucket, MaxKeys=batch_delete_limit).get(resource_name, [])
+
+        items = list_func()
+        while len(items) > 0:
+            keys = list(map(self._map_version_key, items))
+            self.client.delete_objects(Bucket=bucket, Delete={'Objects': keys})
+            items = list_func()
+
     @retry_handler
     def delete_object(self, *args, **kwargs):
         return self.client.delete_object(*args, **kwargs)
@@ -99,7 +118,7 @@ class BotoClient:
         )
         s3_client = boto3.client(
             's3',
-            config=boto3.session.Config(signature_version='s3v4', 
+            config=boto3.session.Config(signature_version='s3v4',
                 region_name=response.get('LocationConstraint'))
         )
 
@@ -113,7 +132,7 @@ class BotoClient:
 
     def get_waiter_names(self):
         return self.client.waiter_names
-        
+
 class S3FSClient:
 
     def _get_relative_path(self, path):
@@ -157,7 +176,7 @@ class S3FSClient:
             logging.error("generate_presigned_url failed: %s"%e)
 
         return None
-            
+
     def wait_for_path(self, path):
         path = self._get_relative_path(path)
         if 'object_exists' in self.client.get_waiter_names():
@@ -274,15 +293,37 @@ class S3FSClient:
         if parent:
             self.create_folder(parent)
 
-    def ensure_bucket_created(self, Bucket):
+    def ensure_bucket_created(self, Bucket, region=None):
         try:
+            self.client = BotoClient(region=region)
             self.client.head_bucket(Bucket=Bucket)
         except botocore.client.ClientError as e:
             if e.response['Error']['Code'] == '404':
-                self.client.create_bucket(Bucket=Bucket)
+                kwargs = {'Bucket': Bucket}
+
+                if region:
+                    kwargs['CreateBucketConfiguration'] = {'LocationConstraint': region}
+
+                self.client.create_bucket(**kwargs)
             else:
                 raise
 
+    def ensure_bucket_deleted(self, Bucket):
+        try:
+            self.client = BotoClient()
+            self.client.head_bucket(Bucket=Bucket)
+            self.client.delete_bucket(Bucket=Bucket)
+        except botocore.client.ClientError as e:
+            code = e.response['Error']['Code']
+            # 404 - deleted, that how it should work
+            # 400 - looks like a bug in S3 API, it's returned for deleted versioned buckets
+            # same as in https://github.com/aws/aws-sdk-js/issues/2749
+            if code == '404' or code == '400':
+                return True
+            else:
+                raise
+
+
     def remove_folder(self, path, remove_self=True):
         path = self._get_relative_path(path)
         self._s3_removeFolder(path, remove_self)
@@ -414,7 +455,7 @@ class S3FSClient:
     def read_text_file(self, path):
         from .local_fsclient import LocalFSClient
 
-        with LocalFSClient().save_atomic(path, move_file=False) as local_tmp_path:
+        with LocalFSClient().save_atomic(path) as local_tmp_path:
             self.download_file(path, local_tmp_path)
             return LocalFSClient().read_text_file(local_tmp_path)
 
@@ -424,13 +465,14 @@ class S3FSClient:
     def write_text_file(self, path, data, atomic=False, mode="w"):
         #TODO: support mode="a"
         path = self._get_relative_path(path)
-        mimetype, _ = mimetypes.guess_type(path)
-        if mimetype is None:
-            self.client.put_object(
-                Body=data, Bucket=self.s3BucketName, Key=path)
-        else:
-            self.client.put_object(
-                Body=data, Bucket=self.s3BucketName, Key=path, ContentType=mimetype)
+        mimetype, encoding = mimetypes.guess_type(path)
+        args = {}
+        if mimetype:
+            args['ContentType'] = mimetype
+        if encoding:
+            args['ContentType'] = "application/octet-stream"
+
+        self.client.put_object(Body=data, Bucket=self.s3BucketName, Key=path, **args)
 
     def copy_file_remote(self, path_src, path_dst):
         path = self._get_relative_path(path_src)
@@ -456,14 +498,16 @@ class S3FSClient:
 
         s3_config = boto3.s3.transfer.TransferConfig(use_threads=False)
 
-        mimetype, _ = mimetypes.guess_type(path_local)
-        if mimetype is None:
-            self.client.upload_file(
-                path_local, Bucket=self.s3BucketName, Key=path_s3, Config=s3_config)
-        else:
-            self.client.upload_file(path_local, Bucket=self.s3BucketName, Key=path_s3, Config=s3_config,
-                                    ExtraArgs={"ContentType": mimetype}
-                                    )
+        mimetype, encoding = mimetypes.guess_type(path_local)
+        args = {}
+        if mimetype:
+            args['ContentType'] = mimetype
+        if encoding:
+            args['ContentType'] = "application/octet-stream"
+
+        self.client.upload_file(path_local, Bucket=self.s3BucketName, Key=path_s3, Config=s3_config,
+            ExtraArgs=args
+        )
 
     def copy_files(self, path_src, path_dst):
         files = self.list_folder(path_src, wild=True)
@@ -495,7 +539,7 @@ class S3FSClient:
 
             uri = urlparse(path)
 
-            with LocalFSClient().save_atomic(uri.path, move_file=False) as temp_file:
+            with LocalFSClient().save_atomic(uri.path) as temp_file:
                 LocalFSClient().downloadFile(path, temp_file)
                 self._s3_upload_file(temp_file, s3_path)
 
