@@ -16,9 +16,11 @@ from a2ml.api.a2ml_experiment import A2MLExperiment
 from a2ml.api.a2ml_project import A2MLProject
 from a2ml.api.model_review.model_helper import ModelHelper
 from a2ml.api.model_review.model_review import ModelReview
+from a2ml.api.stats.feature_divergence import FeatureDivergence
 from a2ml.api.utils import dict_dig, merge_dicts
 from a2ml.api.utils.json_utils import json_dumps_np
 from a2ml.api.utils.context import Context
+from a2ml.api.utils.s3_fsclient import S3FSClient
 from a2ml.tasks_queue.config import Config
 
 from .celery_app import celeryApp
@@ -163,13 +165,21 @@ def _read_hub_experiment_session(ctx, params):
 
     ctx.config.set('experiment/cross_validation_folds',
         evaluation_options.get('crossValidationFolds', 5))
+    split_options = evaluation_options.get('splitOptions', {})
+    if split_options.get('trainRatio'):
+        ctx.config.set('experiment/validation_size', 1.0-float(split_options.get('trainRatio')))
+    if split_options.get('shuffleData'):
+        ctx.config.set('experiment/shuffle', split_options.get('shuffleData'))
+    if split_options.get('timeseriesSplit'):
+        ctx.config.set('experiment/timeseries_fold_split', split_options.get('timeseriesSplit'))
+
     ctx.config.set('experiment/max_total_time',
         evaluation_options.get('max_total_time_mins', 60))
 
-    ctx.config.set('experiment/max_cores_per_iteration',
+    ctx.config.set('experiment/max_cores_per_trial',
         evaluation_options.get('cpu_per_mt_algorithm', 1))
-    ctx.config.set('experiment/max_concurrent_iterations',
-        evaluation_options.get('max_concurrent_iterations', 1))
+    ctx.config.set('experiment/max_concurrent_trials',
+        evaluation_options.get('trials_per_worker', 1))
 
     ctx.config.set('experiment/max_eval_time',
         evaluation_options.get('max_eval_time_mins', 6))
@@ -180,6 +190,13 @@ def _read_hub_experiment_session(ctx, params):
     ctx.config.set('experiment/metric',
         evaluation_options.get('scoring'), provider)
 
+    if evaluation_options.get('algorithms_to_exlude'):
+        ctx.config.set('experiment/blocked_models', evaluation_options.get('algorithms_to_exlude'))
+    if evaluation_options.get('allowed_algorithms'):
+        ctx.config.set('experiment/allowed_models', evaluation_options.get('allowed_algorithms'))
+    if evaluation_options.get('exit_score'):
+        ctx.config.set('experiment/exit_score', evaluation_options.get('exit_score'))
+
     return ctx
 
 def _update_hub_objects(ctx, provider, params):
@@ -272,6 +289,11 @@ def _create_provider_context(params):
         debug=task_config.debug
     )
 
+    if provider == 'auger':
+        auger_creds = params.get('auger_creds', None)
+        if auger_creds:
+            ctx.credentials = auger_creds
+
     ctx.set_runs_on_server(True)
     ctx.config.set('providers', [provider])
 
@@ -449,7 +471,8 @@ def deploy_model_task(params):
     ctx = _read_hub_experiment_session(ctx, params)
 
     ctx.config.clean_changes()
-    res = A2ML(ctx).deploy(model_id = params.get('model_id'), review = params.get('support_review_model'))
+    res = A2ML(ctx).deploy(model_id = params.get('model_id'), review = params.get('support_review_model'),
+        locally=params.get('locally', False))
     _update_hub_objects(ctx, params.get('provider'), params)
 
     return res
@@ -458,6 +481,7 @@ def deploy_model_task(params):
 @process_task_result
 def undeploy_model_task(params):
     ctx = _create_provider_context(params)
+    provider = params.get('provider', 'auger')
     ctx = _read_hub_experiment_session(ctx, params)
 
     model_id = params.get('hub_info', {}).get('pipeline_id')
@@ -467,8 +491,9 @@ def undeploy_model_task(params):
     if not model_id:
         raise Exception("undeploy_model_task: hub_info/pipeline_id should be provided.")
 
+    ctx.config.set('undeploy/service_only', params.get('service_only', False), provider)
     ctx.config.clean_changes()
-    res = A2MLModel(ctx).undeploy(model_id = model_id)
+    res = A2MLModel(ctx).undeploy(model_id = model_id, locally=params.get('locally', False))
     _update_hub_objects(ctx, params.get('provider'), params)
 
     return res
@@ -480,8 +505,8 @@ def predict_by_model_task(params):
 
     ctx = _create_provider_context(params)
     ctx = _read_hub_experiment_session(ctx, params)
-
     ctx.config.clean_changes()
+
     runner = CRUDRunner(ctx, "%s"%params.get('provider'), 'model')
     res = list(runner.providers.values())[0].predict(
         filename=params.get('path_to_predict'),
@@ -492,7 +517,8 @@ def predict_by_model_task(params):
         json_result=params.get('json_result'),
         count_in_result=params.get('count_in_result'),
         predicted_at=params.get('prediction_date'),
-        prediction_id = params.get('prediction_id')
+        prediction_id = params.get('prediction_id'),
+        locally = params.get('locally', False)
     )
     _update_hub_objects(ctx, params.get('provider'), params)
 
@@ -501,36 +527,31 @@ def predict_by_model_task(params):
 @celeryApp.task(ignore_result=True)
 @process_task_result
 def score_actuals_by_model_task(params):
+    ctx = _create_provider_context(params)
+    ctx = _read_hub_experiment_session(ctx, params)
+    ctx.config.clean_changes()
+
     return ModelReview(params).add_actuals(
+        ctx,
         actuals_path = params.get('actuals_path'),
-        actual_records=params.get('actual_records'),
-        prediction_group_id=params.get('prediction_group_id', None),
-        primary_prediction_group_id=params.get('primary_prediction_group_id', None),
-        primary_model_path=ModelHelper.get_model_path(params.get('primary_pipeline_id', None),
-            params.get('hub_info', {}).get('project_path')),
+        data=params.get('actual_records'),
+        columns=params.get('actual_columns', None),
         actual_date=params.get('actual_date'),
+        actual_date_column=params.get('actual_date_column'),
         actuals_id=params.get('actuals_id'),
-        calc_score=params.get('calc_score', True),
-        return_count=params.get('return_count', False)
+        return_count=params.get('return_count', False),
+        provider=params.get('provider')
     )
 
 @celeryApp.task(ignore_result=True)
 @process_task_result
 def delete_actuals_task(params):
     return ModelReview(params).delete_actuals(
-        with_predictions=params.get('with_predictions'), 
-        begin_date=params.get('begin_date'), 
-        end_date=params.get('end_date'), 
+        with_predictions=params.get('with_predictions'),
+        begin_date=params.get('begin_date'),
+        end_date=params.get('end_date'),
     )
 
-# @celeryApp.task(ignore_result=True)
-# @process_task_result
-# def count_actuals_by_prediction_id_task(params):
-#     return ModelReview(params).count_actuals_by_prediction_id(
-#         date_from=params.get('date_from'),
-#         date_to=params.get('date_to')
-#     )
-
 @celeryApp.task(ignore_result=True)
 @process_task_result
 def score_model_performance_daily_task(params):
@@ -567,6 +588,55 @@ def clear_model_results_and_actuals(params):
 @celeryApp.task(ignore_result=True)
 @process_task_result
 def build_review_data_task(params):
+    ctx = _create_provider_context(params)
+    ctx = _read_hub_experiment_session(ctx, params)
+
+    date_col = None
+    if ctx.config.get_list('experiment/date_time'):
+        date_col = ctx.config.get_list('experiment/date_time')[0]
+
     return ModelReview(params).build_review_data(
-        data_path=params.get('data_path')
+        data_path=params.get('data_path'),
+        date_col = date_col
     )
+
+@celeryApp.task(ignore_result=True)
+@process_task_result
+def build_divergence_model_task(params):
+    return FeatureDivergence(params).build_and_save_model()
+
+@celeryApp.task(ignore_result=True)
+@process_task_result
+def calc_divergence_daily_task(params):
+    # In casse of ModelIsNotReadyError try to build it and retry 1 time
+    for attempt in range(2):
+        try:
+            return FeatureDivergence(params).score_divergence_daily(
+                date_from=params.get('date_from'),
+                date_to=params.get('date_to'),
+                top_n=params.get('top_n'),
+            )
+        except FeatureDivergence.ModelIsNotReadyError as e:
+            if attempt == 0:
+                FeatureDivergence(params).build_and_save_model()
+            else:
+                raise e
+
+@celeryApp.task(ignore_result=True)
+@process_task_result
+def create_org_bucket(params):
+    client = S3FSClient()
+    client.ensure_bucket_created(
+        Bucket=params['bucket_name'],
+        region=params.get('region')
+    )
+
+    return True
+
+@celeryApp.task(ignore_result=True)
+@process_task_result
+def delete_org_bucket(params):
+    client = S3FSClient()
+    client.ensure_bucket_deleted(Bucket=params['bucket_name'])
+
+    return True
